@@ -24,9 +24,6 @@
         f3         x
              1           2   3
 """
-import six
-
-
 class IndexFilter:
     """A generic index filter"""
     def __init__(self, func):
@@ -34,6 +31,7 @@ class IndexFilter:
         :param Function func: a evaluation function
         """
         self.eval_func = func
+        # indices to be filtered out
         self.indices = None
 
     def rows(self):
@@ -58,6 +56,7 @@ class IndexFilter:
         original ones before filtering"""
         pass
 
+
 class ColumnIndexFilter(IndexFilter):
     """A column filter that operates on column indices"""
     def columns(self):
@@ -70,9 +69,7 @@ class ColumnIndexFilter(IndexFilter):
 
         :param Matrix reader: a Matrix instance
         """
-        self.indices = filter(self.eval_func, reader.column_range())
-        if six.PY3:
-            self.indices = list(self.indices)
+        self.indices = [ i for i in reader.column_range() if self.eval_func(i) ]
 
     def translate(self, row, column):
         """Map the row, column after filtering to the
@@ -93,8 +90,7 @@ class ColumnIndexFilter(IndexFilter):
 
 
 class ColumnFilter(ColumnIndexFilter):
-    """A column filter that takes in a list of indices to be
-    filtered out""" 
+    """Filters out a list of columns""" 
     def __init__(self, indices):
         """Constructor
 
@@ -104,8 +100,19 @@ class ColumnFilter(ColumnIndexFilter):
         ColumnIndexFilter.__init__(self, eval_func)
 
 
+class SingleColumnFilter(ColumnIndexFilter):
+    """Filters out a single column index""" 
+    def __init__(self, index):
+        """Constructor
+
+        :param list indices: a list of column indices to be filtered out
+        """
+        eval_func = lambda x: x == index
+        ColumnIndexFilter.__init__(self, eval_func)
+
+
 class OddColumnFilter(ColumnIndexFilter):
-    """A column filter that filters out odd indices
+    """Filters out odd indexed columns
 
     * column 0 is regarded as the first column.
     * column 1 is regarded as the seocond column -> this will be filtered out
@@ -116,7 +123,7 @@ class OddColumnFilter(ColumnIndexFilter):
 
 
 class EvenColumnFilter(ColumnIndexFilter):
-    """A column filter that filters out even indices
+    """Filters out even indexed columns
 
     * column 0 is regarded as the first column. -> this will be filtered out
     * column 1 is regarded as the seocond column
@@ -127,7 +134,7 @@ class EvenColumnFilter(ColumnIndexFilter):
 
 
 class RowIndexFilter(IndexFilter):
-    """A row filter that operates on column indices"""
+    """Filter out rows by its row index """
     def rows(self):
         """number of rows to be filtered out"""
         if self.indices:
@@ -141,9 +148,7 @@ class RowIndexFilter(IndexFilter):
 
         :param Matrix reader: a Matrix instance
         """
-        self.indices = filter(self.eval_func, reader.row_range())
-        if six.PY3:
-            self.indices = list(self.indices)
+        self.indices = [ i for i in reader.row_range() if self.eval_func(i) ]
 
     def translate(self, row, column):
         """Map the row, column after filtering to the
@@ -164,8 +169,7 @@ class RowIndexFilter(IndexFilter):
 
 
 class RowFilter(RowIndexFilter):
-    """A row filter that takes in a list of indices to be
-    filtered out""" 
+    """Filters a list of rows""" 
     def __init__(self, indices):
         """Constructor
 
@@ -175,9 +179,19 @@ class RowFilter(RowIndexFilter):
         RowIndexFilter.__init__(self, eval_func)
 
 
+class SingleRowFilter(RowIndexFilter):
+    """Filters out a single row"""
+    def __init__(self, index):
+        """Constructor
+
+        :param list indices: a list of column indices to be filtered out
+        """
+        eval_func = lambda x: x == index
+        RowIndexFilter.__init__(self, eval_func)
+
+
 class OddRowFilter(RowIndexFilter):
-    """
-    Filter out odd rows
+    """Filters out odd indexed rows
 
     row 0 is seen as the first row
     """
@@ -187,8 +201,7 @@ class OddRowFilter(RowIndexFilter):
 
 
 class EvenRowFilter(RowIndexFilter):
-    """
-    Filter out even rows
+    """Filters out even indexed rows
 
     row 0 is seen as the first row
     """
@@ -198,7 +211,10 @@ class EvenRowFilter(RowIndexFilter):
 
 
 class RowValueFilter(RowIndexFilter):
-    """Filter out rows that satisfy a condition"""
+    """Filters out rows based on its row values
+
+    .. note:: it takes time as it needs to go through all values
+    """
     def validate_filter(self, reader):
         """
         Filter out the row indices
@@ -217,6 +233,30 @@ class RowValueFilter(RowIndexFilter):
         self.indices = [row[0] for row in enumerate(reader.rows()) if not self.eval_func(row[1])]
 
 
+class SeriesRowValueFilter(RowIndexFilter):
+    """Filter out rows that satisfy a condition
+
+    .. note:: it takes time as it needs to go through all values
+    """
+    def validate_filter(self, reader):
+        """
+        Filter out the row indices
+
+        This is what it does::
+        
+            new_indices = []
+            index = 0
+            for r in reader.rows():
+                if not self.eval_func(r):
+                    new_indices.append(index)
+                index += 1
+        
+        :param Matrix reader: a Matrix instance
+        """
+        series = reader.colnames
+        self.indices = [row[0] for row in enumerate(reader.rows()) if not self.eval_func(dict(zip(series, row[1])))]
+
+
 class RowInFileFilter(RowValueFilter):
     """Filter out rows that has a row from another reader"""
     def __init__(self, reader):
