@@ -1,3 +1,12 @@
+"""
+    pyexcel.io.csvbook
+    ~~~~~~~~~~~~~~~~~~~
+
+    The lower level csv file format handler.
+
+    :copyright: (c) 2014 by C. W.
+    :license: GPL v3
+"""
 import six
 import csv
 import codecs
@@ -46,7 +55,7 @@ class CSVBook:
             self.mysheets = {"csv":[]}
             # no content let's return'
             return
-        reader = csv.reader(f, dialect=csv.excel, **keywords)
+        reader = csv.reader(f, **keywords)
         longest_row_length = -1
         for row in reader:
             myrow = []
@@ -108,7 +117,8 @@ class CSVSheetWriter:
         """
         This call close the file handle
         """
-        self.f.close()
+        if not isinstance(self.f, StringIO):
+            self.f.close()
 
 
 class CSVWriter:
