@@ -7,13 +7,14 @@
     :copyright: (c) 2014 by C. W.
     :license: GPL v3
 """
+import re
 import six
 import copy
+from ._compact import is_array_type
 
 
-def f7(seq):
-    """
-    Return a unique list of the incoming list
+def _unique(seq):
+    """Return a unique list of the incoming list
     
     Reference:
     http://stackoverflow.com/questions/480214/
@@ -25,8 +26,7 @@ def f7(seq):
 
 
 def longest_row_number(array):
-    """
-    Find the length of the longest row in the array
+    """Find the length of the longest row in the array
 
     :param list in_array: a list of arrays    
     """
@@ -38,8 +38,7 @@ def longest_row_number(array):
 
 
 def uniform(array):
-    """
-    Fill-in empty strings to empty cells to make it MxN
+    """Fill-in empty strings to empty cells to make it MxN
 
     :param list in_array: a list of arrays
     """
@@ -53,12 +52,18 @@ def uniform(array):
                 row += [""] * (width - row_length)
         return array
 
-
 def transpose(in_array):
-    """
-    Rotate the array by 90 degrees
+    """Rotate clockwise by 90 degrees and flip horizontally
 
+    First column become first row. 
     :param list in_array: a list of arrays
+
+    The transformation is::
+
+        1 2 3       1  4
+        4 5 6 7 ->  2  5
+                    3  6
+                    '' 7
     """
     max_length = longest_row_number(in_array)
     new_array = []
@@ -72,10 +77,63 @@ def transpose(in_array):
         new_array.append(row_data)
     return new_array
 
+"""
+In order to easily compute the actual index of 'X' or 'AX', these utility
+functions were written
+"""
+
+_INDICES = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+
+def _get_index(index_chars):
+    length = len(index_chars)
+    if len(index_chars) > 1:
+        return (_get_index(index_chars[0])+1) * (len(_INDICES) ** (length-1)) + _get_index(index_chars[1:])
+    else:
+        return _INDICES.index(index_chars[0])
+
+def _excel_column_index(index_chars):
+    if len(index_chars) < 1:
+        return -1
+    else:
+        return _get_index(index_chars.upper())
+
+def _excel_cell_position(pos_chars):
+    if len(pos_chars) < 2:
+        return -1, -1
+    group = re.match("([A-Za-z]+)([0-9]+)", pos_chars)
+    if group:    
+        return int(group.group(2)) - 1, _excel_column_index(group.group(1))
+    else:
+        raise IndexError
+    
+def _analyse_slice(aslice, upper_bound):
+    """An internal function to analyze a given slice
+    """
+    if aslice.start is None:
+        start = 0
+    else:
+        start = max(aslice.start, 0)
+    if aslice.stop is None:
+        stop = upper_bound
+    else:
+        stop = min(aslice.stop, upper_bound)
+    if start > stop:
+        raise ValueError
+    elif start < stop:
+        if aslice.step:
+            my_range = range(start, stop, aslice.step)
+        else:
+            my_range = range(start, stop)
+        if six.PY3:
+            # for py3, my_range is a range object
+            my_range = list(my_range)
+    else:
+        my_range = [start]
+    return my_range
+
 
 class Row:
-    """
-    Represet row of a matrix
+    """Represet row of a matrix
 
     .. table:: "example.csv"
 
@@ -86,49 +144,28 @@ class Row:
         = = =
     
     Above column manipluation can be performed on rows similiarly. This section will not repeat the same example but show some advance usages. 
+
     
         >>> import pyexcel as pe
-        >>> reader = pe.Reader("example.csv")
-        >>> reader.row[0:2]
-        [[1, 4, 7], [2, 5, 8]]
-        >>> reader.row[0:3] = [0, 0, 0]
-        >>> reader.row[2]
+        >>> data = [[1,2,3], [4,5,6], [7,8,9]]
+        >>> m = pe.iterators.Matrix(data)
+        >>> m.row[0:2]
+        [[1, 2, 3], [4, 5, 6]]
+        >>> m.row[0:3] = [0, 0, 0]
+        >>> m.row[2]
         [0, 0, 0]
-        >>> del reader.row[0:2]
-        >>> reader.row[0]
+        >>> del m.row[0:2]
+        >>> m.row[0]
         [0, 0, 0]
 
     """
     def __init__(self, matrix):
         self.ref = matrix
 
-    def _analyse_slice(self, aslice):
-        if aslice.start is None:
-            start = 0
-        else:
-            start = max(aslice.start, 0)
-        if aslice.stop is None:
-            stop = self.ref.number_of_rows()
-        else:
-            stop = min(aslice.stop, self.ref.number_of_rows())
-        if start > stop:
-            raise ValueError
-        elif start < stop:
-            if aslice.step:
-                my_range = range(start, stop, aslice.step)
-            else:
-                my_range = range(start, stop)
-            if six.PY3:
-                # for py3, my_range is a range object
-                my_range = list(my_range)
-        else:
-            my_range = [start]
-        return my_range
-
     def __delitem__(self, aslice):
         """Override the operator to delete items"""
         if isinstance(aslice, slice):
-            my_range = self._analyse_slice(aslice)
+            my_range = _analyse_slice(aslice, self.ref.number_of_rows())
             self.ref.delete_rows(my_range)
         else:
             self.ref.delete_rows([aslice])
@@ -136,7 +173,7 @@ class Row:
     def __setitem__(self, aslice, c):
         """Override the operator to set items"""
         if isinstance(aslice, slice):
-            my_range = self._analyse_slice(aslice)
+            my_range = _analyse_slice(aslice, self.ref.number_of_rows())
             for i in my_range:
                 self.ref.set_row_at(i, c)
         else:
@@ -147,7 +184,7 @@ class Row:
         from left to right"""
         index = aslice
         if isinstance(aslice, slice):
-            my_range = self._analyse_slice(aslice)
+            my_range = _analyse_slice(aslice, self.ref.number_of_rows())
             results = []
             for i in my_range:
                 results.append(self.ref.row_at(i))
@@ -180,8 +217,7 @@ class Row:
 
 
 class Column:
-    """
-    Represet columns of a matrix
+    """Represet columns of a matrix
     
     .. table:: "example.csv"
 
@@ -194,48 +230,33 @@ class Column:
     Let us manipulate the data columns on the above data matrix::
     
         >>> import pyexcel as pe
-        >>> reader = pe.Reader("example.csv")
-        >>> reader.column[0]
+        >>> data = [[1,2,3], [4,5,6], [7,8,9]]
+        >>> m = pe.iterators.Matrix(data)
+        >>> m.column[0]
         [1, 4, 7]
-        >>> reader.column[2] = [0, 0, 0]
-        >>> reader.column[2]
+        >>> m.column[2] = [0, 0, 0]
+        >>> m.column[2]
         [0, 0, 0]
-        >>> del reader.column[2]
-        >>> reader.column[2]
-        [3, 6, 9]
+        >>> del m.column[1]
+        >>> m.column[1]
+        [0, 0, 0]
+        >>> m.column[2]
+        Traceback (most recent call last):
+            ...
+        IndexError
 
     """
     def __init__(self, matrix):
         self.ref = matrix
 
-    def _analyse_slice(self, aslice):
-        if aslice.start is None:
-            start = 0
-        else:
-            start = max(aslice.start, 0)
-        if aslice.stop is None:
-            stop = self.ref.number_of_columns()
-        else:
-            stop = min(aslice.stop, self.ref.number_of_columns())
-        if start > stop:
-            raise ValueError
-        elif start < stop:
-            if aslice.step:
-                my_range = range(start, stop, aslice.step)
-            else:
-                my_range = range(start, stop)
-            if six.PY3:
-                # for py3, my_range is a range object
-                my_range = list(my_range)
-        else:
-            my_range = [start]
-        return my_range
-
     def __delitem__(self, aslice):
         """Override the operator to delete items"""
         if isinstance(aslice, slice):
-            my_range = self._analyse_slice(aslice)
+            my_range = _analyse_slice(aslice, self.ref.number_of_columns())
             self.ref.delete_columns(my_range)
+        elif isinstance(aslice, str):
+            index = _excel_column_index(aslice)
+            self.ref.delete_columns([index])
         elif isinstance(aslice, int):
             self.ref.delete_columns([aslice])
         else:
@@ -244,9 +265,12 @@ class Column:
     def __setitem__(self, aslice, c):
         """Override the operator to set items"""
         if isinstance(aslice, slice):
-            my_range = self._analyse_slice(aslice)
+            my_range = _analyse_slice(aslice, self.ref.number_of_columns())
             for i in my_range:
                 self.ref.set_column_at(i, c)
+        elif isinstance(aslice, str):
+            index = _excel_column_index(aslice)
+            self.ref.set_column_at(index, c)
         elif isinstance(aslice, int):
             self.ref.set_column_at(aslice, c)
         else:
@@ -257,11 +281,13 @@ class Column:
         from left to right"""
         index = aslice
         if isinstance(aslice, slice):
-            my_range = self._analyse_slice(aslice)
+            my_range = _analyse_slice(aslice, self.ref.number_of_columns())
             results = []
             for i in my_range:
                 results.append(self.ref.column_at(i))
             return results
+        elif isinstance(aslice, str):
+            index = _excel_column_index(aslice)
         if index in self.ref.column_range():
             return self.ref.column_at(index)
         else:
@@ -275,7 +301,7 @@ class Column:
         if isinstance(other, list):
             self.ref.extend_columns(other)
         elif isinstance(other, Matrix):
-            self.ref.extend_columns(other.array)
+            self.ref.extend_columns_with_rows(other.array)
         else:
             raise TypeError
         return self
@@ -290,9 +316,7 @@ class Column:
 
 
 class Matrix:
-    """
-    The internal representation of a sheet data. Each element
-    can be of any python types
+    """The internal representation of a sheet data. Each element can be of any python types
     """
     
     def __init__(self, array):
@@ -360,21 +384,18 @@ class Matrix:
 
     def enumerate(self):
         """
-        Default iterator to go through each cell one by one from top row to
-        bottom row and from left to right
-        example::
+        Iterate cell by cell from top to bottom and from left to right
+
+        .. testcode::
     
-            import pyexcel as pe
-            data = [  
-                [1, 2, 3, 4],
-                [5, 6, 7, 8],
-                [9, 10, 11, 12]
-            ]
-            m = pe.Matrix(data)
-            print(pe.utils.to_array(m.enumerate()))
-    
-        output::
-        
+            >>> import pyexcel as pe
+            >>> data = [  
+            ...     [1, 2, 3, 4],
+            ...     [5, 6, 7, 8],
+            ...     [9, 10, 11, 12]
+            ... ]
+            >>> m = pe.iterators.Matrix(data)
+            >>> print(pe.utils.to_array(m.enumerate()))
             [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
 
         More details see :class:`HTLBRIterator`
@@ -382,22 +403,20 @@ class Matrix:
         return HTLBRIterator(self)
 
     def reverse(self):
-        """
-        Reverse iterator to go through each cell one by one from
+        """Opposite to enumerate
+
+        each cell one by one from
         bottom row to top row and from right to left
         example::
     
-            import pyexcel as pe
-            data = [  
-                [1, 2, 3, 4],
-                [5, 6, 7, 8],
-                [9, 10, 11, 12]
-            ]
-            m = pe.Matrix(data)
-            print(pe.utils.to_array(m.reverse()))
-    
-        output::
-        
+            >>> import pyexcel as pe
+            >>> data = [  
+            ...     [1, 2, 3, 4],
+            ...     [5, 6, 7, 8],
+            ...     [9, 10, 11, 12]
+            ... ]
+            >>> m = pe.iterators.Matrix(data)
+            >>> print(pe.utils.to_array(m.reverse()))
             [12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1]
 
         More details see :class:`HBRTLIterator`
@@ -452,7 +471,7 @@ class Matrix:
 
     def rows(self):
         """
-        Returns a row iterator to go through each row from top to bottom
+        Returns a top to bottom row iterator
 
         example::
     
@@ -475,9 +494,9 @@ class Matrix:
 
     def rrows(self):
         """
-        Returns a row iterator to go through each row from bottom to top
+        Returns a bottom to top row iterator
         
-        example::
+        .. testcode::
     
             import pyexcel as pe
             data = [  
@@ -488,7 +507,7 @@ class Matrix:
             m = pe.Matrix(data)
             print(pe.utils.to_array(m.rrows()))
     
-        output::
+        .. testoutput::
         
             [[9, 10, 11, 12], [5, 6, 7, 8], [1, 2, 3, 4]]
 
@@ -498,8 +517,9 @@ class Matrix:
 
     def columns(self):
         """
-        Returns a column iterator to go through each column from left to right
-        example::
+        Returns a left to right column iterator
+      
+        .. testcode::
     
             import pyexcel as pe
             data = [  
@@ -510,7 +530,7 @@ class Matrix:
             m = pe.Matrix(data)
             print(pe.utils.to_array(m.columns()))
     
-        output::
+        .. testoutput::
         
             [[1, 5, 9], [2, 6, 10], [3, 7, 11], [4, 8, 12]]
 
@@ -520,7 +540,7 @@ class Matrix:
 
     def rcolumns(self):
         """
-        Returns a column iterator to go through each column from right to left
+        Returns a right to left column iterator
     
         example::
     
@@ -563,7 +583,7 @@ class Matrix:
 
     def row_at(self, index):
         """
-        Returns an array that collects all data at the specified row
+        Gets the data at the specified row
         """
         if index in self.row_range():
             cell_array = []
@@ -571,11 +591,11 @@ class Matrix:
                 cell_array.append(self.cell_value(index, i))
             return cell_array
         else:
-            return None
+            raise IndexError("Out of range")
 
     def column_at(self, index):
         """
-        Returns an array that collects all data at the specified column
+        Gets the data at the specified column
         """
         if index in self.column_range():
             cell_array = []
@@ -583,10 +603,10 @@ class Matrix:
                 cell_array.append(self.cell_value(i, index))
             return cell_array
         else:
-            return None
+            raise IndexError("Out of range")
 
     def set_column_at(self, column_index, data_array, starting=0):
-        """Update columns
+        """Updates a column data range
 
         It works like this if the call is: set_column_at(2, ['N','N', 'N'], 1)::
 
@@ -613,7 +633,7 @@ class Matrix:
             raise IndexError
 
     def set_row_at(self, row_index, data_array, starting=0):
-        """Update rows
+        """Update a row data range
 
         It works like this if the call is: set_row_at(2, ['N', 'N', 'N'], 1)::
 
@@ -638,38 +658,35 @@ class Matrix:
         else:
             raise IndexError
 
+    def _extend_row(self, row):
+        array = copy.deepcopy(row)
+        self.array.append(array)
+
     def extend_rows(self, rows):
-        """expected the rows to be off the same length"""
-        array_length = self.number_of_columns()
-        max_length = array_length
-        for r in rows:
-            length = len(r)
-            agreed_length = max(length, array_length)
-            if max_length < agreed_length:
-                max_length = agreed_length
-            array = copy.deepcopy(r[:agreed_length])
-            if length < array_length:
-                array = array + [""] * (array_length-length)
-            self.array.append(array)
-        # if number_of_rows > 0, means self has content
-        # if self does not have content, does not make sense to
-        # to increase width
-        self.array = uniform(self.array)
+        """Inserts two dimensinal data after the bottom row"""
+        if isinstance(rows, list):
+            if is_array_type(rows, list):
+                for r in rows:
+                    self._extend_row(r)
+            else:
+                self._extend_row(rows)
+            self.array = uniform(self.array)
+        else:
+            raise TypeError("Cannot use %s" % type(rows))
 
     def delete_rows(self, row_indices):
-        """delete rows by specified row indices"""
+        """Deletes specified row indices"""
         if isinstance(row_indices, list) is False:
             raise IndexError
         if len(row_indices) > 0:
-            unique_list = f7(row_indices)
+            unique_list = _unique(row_indices)
             sorted_list = sorted(unique_list, reverse=True)
             for i in sorted_list:
                 if i < self.number_of_rows():
                     del self.array[i]
 
     def extend_columns(self, columns):
-        """
-        columns should be an array
+        """Inserts two dimensional data after the rightmost column
 
         This is how it works:
 
@@ -681,30 +698,40 @@ class Matrix:
         
             s s s  +  t t
         """
+        if not isinstance(columns, list):
+            raise TypeError("Cannot extend using type %s" % type(columns))
+        incoming_data = columns
+        if not is_array_type(columns, list):
+            incoming_data = [columns]
+        incoming_data = transpose(incoming_data)
+        self._extend_columns_with_rows(incoming_data)    
+
+    def _extend_columns_with_rows(self, rows):
         current_nrows = self.number_of_rows()
         current_ncols = self.number_of_columns()
-        insert_column_nrows = len(columns)
+        insert_column_nrows = len(rows)
         array_length = min(current_nrows, insert_column_nrows)
         for i in range(0, array_length):
-            array = copy.deepcopy(columns[i])
+            array = copy.deepcopy(rows[i])
             self.array[i] += array
         if current_nrows < insert_column_nrows:
             delta = insert_column_nrows - current_nrows
             base = current_nrows
             for i in range(0, delta):
                 new_array = [""] * current_ncols
-                new_array += columns[base+i]
+                new_array += rows[base+i]
                 self.array.append(new_array)
         self.array = uniform(self.array)
 
+    def extend_columns_with_rows(self, rows):
+        self._extend_columns_with_rows(rows)
     def delete_columns(self, column_indices):
-        """
-        Delete columns by specified list of indices
+        """Delete columns by specified list of indices
         """
         if isinstance(column_indices, list) is False:
             raise ValueError
         if len(column_indices) > 0:
-            unique_list = f7(column_indices)
+            unique_list = _unique(column_indices)
             sorted_list = sorted(unique_list, reverse=True)
             for i in range(0, len(self.array)):
                 for j in sorted_list:
@@ -714,6 +741,9 @@ class Matrix:
         """Override the operator to set items"""
         if isinstance(aset, tuple):
             return self.cell_value(aset[0], aset[1], c)
+        elif isinstance(aset, str):
+            row, column = _excel_cell_position(aset)
+            return self.cell_value(row, column, c)
         else:
             raise IndexError
 
@@ -722,6 +752,9 @@ class Matrix:
         from left to right"""
         if isinstance(aset, tuple):
             return self.cell_value(aset[0], aset[1])
+        elif isinstance(aset, str):
+            row, column = _excel_cell_position(aset)
+            return self.cell_value(row, column)
         elif isinstance(aset, int):
             print("Deprecated usage. Please use [row, column]")
             return self.row_at(aset)
@@ -743,6 +776,11 @@ class Matrix:
         """
         self.array = transpose(self.array)
 
+    def to_array(self):
+        """Get an array out
+        """
+        return self.array
+
 
 class PyexcelIterator:
     """
@@ -965,11 +1003,7 @@ class HBLTRIterator(VBLTRIterator):
     """
     Horizontal Bottom Left to Top Right Iterator
 
-    Iterate horizontally from bottom left to top right::
-    
-        >>E
-        >>> ^
-        S>> |
+    Iterate horizontally from bottom left to top right
     """
     def __init__(self, reader):
         self.reader_ref = reader
@@ -1076,7 +1110,7 @@ class ColumnReverseIterator(PyexcelIterator):
             raise StopIteration
 
 
-class SeriesColumnIterator(PyexcelIterator):
+class ColumnIndexIterator(PyexcelIterator):
     """
     Column Iterator
 
@@ -1086,7 +1120,6 @@ class SeriesColumnIterator(PyexcelIterator):
     def __init__(self, reader):
         self.reader_ref = reader
         self.current = 0
-        self.headers = reader.series()
 
     def __iter__(self):
         return self
@@ -1095,12 +1128,36 @@ class SeriesColumnIterator(PyexcelIterator):
         if self.current in self.reader_ref.column_range():
             index = self.current
             self.current += 1
-            column_header = self.headers[index]
+            column_header = self.reader_ref.colnames[index]
             return {column_header: self.reader_ref.named_column_at(column_header)}
         else:
             raise StopIteration
 
 
+class RowIndexIterator(PyexcelIterator):
+    """
+    Row Iterator
+
+    Default iterator for :class:`Sheet` when it becomes Series
+    See :func:`Sheet.__iter__` for more details
+    """
+    def __init__(self, reader):
+        self.reader_ref = reader
+        self.current = 0
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        if self.current in self.reader_ref.row_range():
+            index = self.current
+            self.current += 1
+            column_header = self.reader_ref.rownames[index]
+            return {column_header: self.reader_ref.named_row_at(column_header)}
+        else:
+            raise StopIteration
+
+
 class SheetIterator(PyexcelIterator):
     """
     Sheet Iterator
